; ENTMKR.LSP   - C:ENTMKR  - make entmake descriptions from DWG files

; This program creates what I call a DXG file which is composed of entmake
; descriptions of all entities in the drawing. I find that he DXG files take
; considerably less space than the equivalent drawing file. They are also more
; flexible as they can easily be combined. My most common use of this feature
; is that I create complex title blocks which are one single block from
; several DXG files. You can therefore easily keep different drawing sizes,
; client logos or names, and easily combine them with your own title block.

; This program creates 2 files. The user selects a filename for the final DXG
; and the program creates a TMP file with the unfiltered entget information.
; This info is sorted and grouped for LINES, POLYLINES, TEXT, ATTDEF, SOLIDS
; and POINTS. Other entities are read and transferred directly to the DXG file.

; Requires Release 12.

; FUNCTIONS CALLED
; (diddle fun) - show rotating line to indicate activity
; (pretty string counter filler_string flag) - align strings
; (assoc1_sort list) - sort an association list by number
; (instr str1 str2)  - returns location of str2 in str1 or nil
; (stres2 str1 str2) - returns part of str1 ending before str2
; (dxf code elist)   - returns 2nd item of dotted pair associated with code


(defun C:ENTMKR (/ ajhlst ajvlst angflg assitm atalst atclst atdlst
           atflst atglst athlst atjlst atllst atlsty atplst atslst
           attflg atvlst atwlst blgchk blgflg 
           chkang chkcol chkdef chkht chkjh chkjp chklay 
           chklst chksty chkvf chkwd cl colflg ct1 ct2 cvchk 
           cvflg def defflg dxfno ed en endln enm ep
           fi flgchk flgflg flglst fo ht htflg itm
           jhflg jp jpflg l1 lay layflg len llen lnclst lnelst
           lnflg lnllst lnslst lst lstcnt lstfl ly maplst
           n1 n2 natd nedd nil nitem nlin nm nn npol npr2 npt
           nsol nst nstart ntxt of
           p1 p2 p3 p4 plflg pllst plvlst pr1 pr2 prm prom
           ptclst ptflg ptllst ptprc ptslst pvlst rlprc
           soclst solflg sollst sp sp1lst sp2lst sp3lst sp4lst
           str str1 str2 str3 strg sty styflg subpvl
           t1chk t1flg t2chk t2flg tag tjhlst tjvlst tmp tot
           txalst txclst txhlst txjlst txlen txllst txlsty
           txslst txt txtflg txtlst txwlst
           vcflst vfflg vfllst vrdlst vt1lst vt2lst 
           vtflg vtlst vtplst wdflg
          )
 (setq
  nn 
  (getfiled "DXG File to Create" (strcat (getvar "DWGNAME") ".DXG") "DXG" 1)
 ) 
 (if (not nn)   ;quit if no filename has been selected else continue
  (prompt "\nNo file selected! Exiting Program.")
  (progn
   (setq
    rlprc (uint 1 "" "Enter precision for real numbers" 3)
    ptprc (uint 1 "" "Enter precision for points      " 3)
    nm (strcat (substr nn 1 (instr nn ".")) "TMP")
    fo (open nm "w")
    en (entnext) ;PSPACE VIEWPORT  
    fun 0
    nlin 0
    npol 0
    ntxt 0
    natd 0
    nsol 0
    npt  0
    cnt  0
   )
   (prompt (strcat "\nWriting preliminary entmake lists to " nm "\n"))
   (while (setq en (entnext en))  ;step through database
    (setq fun (diddle fun))       
    (setq
     ed (entget en)
     cnt (1+ cnt)
     nedd '()
    )
    (princ (pretty (itoa cnt) 6 " " nil))
    (foreach assitm ed  ;sort and discard unneeded entget info
     (setq
      pr1 (car assitm)
      pr2 (cdr assitm)
      nitem
      (cond
       ((member pr1 '(-1 -2 5)) nil)   ;entity name,handle
       ((member pr1 '(0 1 2 3 6 7 8)) (cons pr1 (strcat "\"" pr2 "\""))) ;string
       ((member pr1 '(10 11 12 13 14 15 16 17 18))
        (setq 
         ptx (atof (rtos (car   pr2) 2 ptprc))
         pty (atof (rtos (cadr  pr2) 2 ptprc))
         ptz (atof (rtos (caddr pr2) 2 ptprc))
         ptz (if (= ptz 0.0) 0 ptz)
         assitm (cons pr1 (list ptx pty ptz))
       ))
       ((member pr1 '(40 41 42 43 44 45 50 51 52 53));real
        (if (/= pr2 0.0) 
         (setq 
          pr2 (atof (rtos pr2 2 rlprc))
          assitm (cons pr1 pr2) 
         )
         nil
       ))
       ((member pr1 '(62 66 73 74 75)) (if (/= pr2 0) assitm nil))      ;integer
       ((= pr1 67)                                                      ;integer
        (if (and (not (member '(0 . "POLYLINE") ed)) (/= pr2 0))
         assitm nil
       ))
       ((member pr1 '(70 71 72))                                        ;integer
        (if
         (or (member '(0 . "POLYLINE") ed)(member '(0 . "ATTDEF") ed)(/= pr2 0))
         assitm nil
       ))
       ((and (= pr1 210) (equal pr2 '(0.0 0.0 1.0))) nil)
       ((= pr1 0) assitm)
       (T assitm)
     ))
     (if nitem (setq nedd (cons nitem nedd)))
    )
    (princ "(entmake '" fo)
    (princ (assoc1-sort nedd) fo)  ; print sorted entget list to file
    (write-line ")" fo)            ; close entmake
    (cond                          ; count entity types to be sorted
     ((instr (setq enm (dxf 0 nedd)) "POLYLINE") (setq npol (1+ npol)))
     ((instr enm "LINE")   (setq nlin (1+ nlin)))
     ((instr enm "TEXT")   (setq ntxt (1+ ntxt)))
     ((instr enm "ATTDEF") (setq natd (1+ natd)))
     ((instr enm "SOLID")  (setq nsol (1+ nsol)))
     ((instr enm "POINT")  (setq npt  (1+ npt)))
    )
   )
   (setq
    fo (close fo)      ;close temp.
    fi (open nm "r")   ;open temp. file for input
    fo (open nn "w")   ;open DXG   file for output
    tot cnt
    cnt 0
   )
   (prompt (strcat "\rCreating entmake lists and writing them to " nn "\n"))
   (while (setq str (read-line fi))  ; read TMP file and create lists for
    (setq                            ; entmake functions for selected entity
     fun (diddle fun)                ; types. Others are written directly
     fl nil                          ; to DXG file.
     cnt (1+ cnt)
    )
    (princ (strcat (pretty (itoa cnt) 6 " " nil) " of " (itoa tot)))
    (cond
     ((and (instr str "TEXT") (> ntxt 1))
      (princ "      TEXT")
      (setq
       txt (strcat "\"" (xtrdxf str 1) "\"")
       txtlst (append txtlst (list txt))
       sty (strcat "\"" (xtrdxf str 7) "\"")
       txlsty (append txlsty (list sty))
       lay (strcat "\"" (xtrdxf str 8) "\"")
       txllst (append txllst (list lay))
       sp (xtrdxf str 10)
       txslst (append txslst (list sp))
       jp (xtrdxf str 11)
       txjlst (append txjlst (list jp))
       ht (xtrdxf str 40)
       txhlst (append txhlst (list ht))
       txwlst
       (append txwlst
        (if (setq l1 (instr str "(41 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(1.0)
       ))
       txalst
       (append txalst
        (if (setq l1 (instr str "(50 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0.0)
       ))
       tjhlst
       (append tjhlst
        (if (setq l1 (instr str "(72 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       tjvlst
       (append tjvlst
        (if (setq l1 (instr str "(73 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       txtflg T
     ))
     ((and (instr str "ATTDEF") (> natd 1))
      (princ "    ATTDEF")
      (setq
       def (strcat "\"" (xtrdxf str 1) "\"")
       atdlst (append atdlst (list def))
       tag (strcat "\"" (xtrdxf str 2) "\"")
       atglst (append atglst (list tag))
       prm (strcat "\"" (xtrdxf str 3) "\"")
       atplst (append atplst (list prm))
       sty (strcat "\"" (xtrdxf str 7) "\"")
       atlsty (append atlsty (list sty))
       lay (strcat "\"" (xtrdxf str 8) "\"")
       atllst (append atllst (list lay))
       sp (xtrdxf str 10)
       atslst (append atslst (list sp))
       jp (xtrdxf str 11)
       atjlst (append atjlst (list jp))
       ht (xtrdxf str 40)
       athlst (append athlst (list ht))
       atwlst
       (append atwlst
        (if (setq l1 (instr str "(41 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(1.0)
       ))
       atalst
       (append atalst
        (if (setq l1 (instr str "(50 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0.0)
       ))
       atvlst
       (append atvlst
        (if (setq l1 (instr str "(70 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       ajhlst
       (append ajhlst
        (if (setq l1 (instr str "(72 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       ajvlst
       (append ajvlst
        (if (setq l1 (instr str "(74 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       attflg T
     ))
     ((and (instr str "SOLID") (> nsol 1))
      (princ "     SOLID")
      (setq
       ly (strcat "\"" (xtrdxf str 8) "\"")
       sollst (append sollst (list ly))
       p1 (xtrdxf str 10)
       sp1lst (append sp1lst (list p1))
       p2 (xtrdxf str 11)
       sp2lst (append sp2lst (list p2))
       p3 (xtrdxf str 12)
       sp3lst (append sp3lst (list p3))
       p4 (xtrdxf str 13)
       sp4lst (append sp4lst (list p4))
       cl (if (instr str "(62 .") (xtrdxf str 62) 256)
       soclst (append soclst (list cl))
       solflg T
     ))                                                     
     ((and (instr str "POLYLINE") (> npol 1))
      (princ "  POLYLINE")
      (setq
       pllst (read (substr str 11 (- (strlen str) 11)))
       ly (strcat "\"" (xtrdxf str 8) "\"")
       pllst
       (subst (cons 0 "\"POLYLINE\"") (assoc 0 pllst)
        (subst (cons 8 ly) (assoc 8 pllst) pllst)
       )
       pllst
       (if (assoc 67 pllst) (subst (cons 67 0) (assoc 67 pllst) pllst) pllst)
       vtplst (list)     ; point
       vt1lst (list)     ; thickness start
       vt2lst (list)     ; thickness end
       vrdlst (list)     ; bulge
       vcflst (list)     ; curve fit flag
       vfllst (list)     ; flag 70
       plflg  T
     ))
     ((and (instr str "VERTEX") (> npol 1))
      (princ "    VERTEX")
      (setq
       sp (xtrdxf str 10)
       vtplst (append vtplst (list sp))
       vt1lst
       (append vt1lst
        (if (setq l1 (instr str "(40 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       vt2lst
       (append vt2lst
        (if (setq l1 (instr str "(41 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       vrdlst
       (append vrdlst
        (if (setq l1 (instr str "(42 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       vcflst
       (append vcflst
        (if (setq l1 (instr str "(50 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
       ))
       vfllst
       (append vfllst
        (if (setq l1 (instr str "(70 "))
         (list (cdr (read (strcat (stres2 (substr str l1) ")") ")"))))
        '(0)
     ))))
     ((and (instr str "SEQEND") (> npol 1))
      (princ "    SEQEND")
      (setq
       plvlst
       (cons (list pllst vtplst vt1lst vt2lst vrdlst vcflst vfllst) plvlst)
     ))
     ((and (not (instr str "YLINE")) (instr str "LINE") (> nlin 1))
      (princ "      LINE")
      (setq
       ly (strcat "\"" (xtrdxf str 8) "\"")
       lnllst (append lnllst (list ly))
       sp (xtrdxf str 10)
       lnslst (append lnslst (list sp))
       ep (xtrdxf str 11)
       lnelst (append lnelst (list ep))
       cl (if (instr str "(62 .") (xtrdxf str 62) 256)
       lnclst (append lnclst (list cl))
       lnflg T
     ))
     ((and (instr str "POINT") (> npt 1))
      (princ "     POINT")
      (setq
       ly (strcat "\"" (xtrdxf str 8) "\"")
       ptllst (append ptllst (list ly))
       sp (xtrdxf str 10)
       ptslst (append ptslst (list sp))
       cl (if (instr str "(62 .") (xtrdxf str 62) 256)
       ptclst (append ptclst (list cl))
       ptflg T
     ))
     (T
      (princ "     OTHER")
      (if (< (strlen (setq str1 (substr str 11 (- (strlen str) 11)))) 65)
       (write-line str fo)
       (progn
        (princ "(entmake\n'" fo)
        (while str1
         (setq
          str2 str1
          fl T
         )
         (while (and (setq len (instr str2 ")")) (< len 60)) ;find last ")" before 60
          (setq
           str2 ""    ;str2 is a dummy str which is created each time of len "*"'s
           str2       ;and then tested for the next ")"
           (strcat
           (repeat len (setq str2 (strcat str2 "*"))) (substr str1 (1+ len))
         )))
         (setq
          str3 (strcat " " (substr str1 (1+ len)))
          str2 (substr str1 1 len)
          str1 nil
         )
         (write-line str2 fo)
         (if (> (strlen str3) 60)
          (setq str1 str3)
          (if
           (> (strlen str3) 3)
           (write-line (substr str3 1 (1- (strlen str3))) fo)
        )))
        (write-line "))" fo)
   )))))
   (setq fi (close fi))
   (prompt
    (strcat "\rReading TMP file done. Sort & write entmake lists to " nn "\n")
   )
   (write-line "(prompt \"--START OF FILE--\")\n" fo)
   (if lnflg
    (progn
     (write-line "\n(prompt \"\\nSTART LINE   LISTS--\")\n" fo)
     (prompt (strcat "\rWriting  LINE START & END POINTS to " nn "\n"))
     (setq
      llen (1- (length lnllst))
      chklay (nth 0 lnllst)  cnt 0  layflg T
     )
     (while (and layflg (< (setq cnt (1+ cnt)) llen))
      (setq layflg (= (nth cnt lnllst) chklay))
     )
     (setq chkcol (nth 0 lnclst)  cnt 0  colflg T)
     (while (and colflg (< (setq cnt (1+ cnt)) llen))
      (setq colflg (= (nth cnt lnclst) chkcol))
     )
     (write-line "(mapcar" fo)
     (write-line
      (strcat
       "'(lambda (" (if layflg "" "ly ") "sp ep" (if colflg "" " cl") ")"
      ) fo
     )
     (write-line "  (entmake" fo)
     (write-line
      (strcat
       "   (list '(0 . \"LINE\") "
       (if layflg (strcat "'(8 . " chklay ")") "(cons 8 ly)")
       " (cons 10 sp) (cons 11 ep) "
       (if colflg (strcat "'(62 . " (itoa chkcol) ")") "(cons 62 cl)")
       ")\n ))"
      )
      fo
     )
     (setq
      maplst (if layflg nil (list lnllst))
      maplst (append maplst (list lnslst) (list lnelst))
      maplst (if colflg maplst (append maplst (list lnclst)))
     )
     (mapcar
     '(lambda (lst)
       (setq cnt -1  n 0  txlen 3)
       (princ "'(" fo)
       (while (setq itm (nth (setq cnt (1+ cnt)) lst))
        (setq fun (diddle fun))
        (if (and (/= cnt 0) (= n 0)) (princ " " fo))
        (cond
         ((= lst lnllst)
          (setq n (1+ n))
          (setq txlen (+ txlen (strlen itm) 3))
          (if (< txlen 80)
           (princ (strcat (if (/= cnt 0) " " "") itm) fo)
           (progn
            (princ (strcat "\n  " itm) fo)
            (setq txlen (+ 2 (strlen itm)))
         )))
         (T
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq n (1+ n))
          (if (or (and (= n 16) (= lst lnclst)) (and (= n 3) (/= lst lnclst)))
           (progn
            (if (/= cnt llen) (write-line "" fo))
            (setq n 0)
       )))))
       (write-line (if (= n cnt) ")" "\n )") fo)
      )
      maplst
     )
     (write-line ")\n\n(prompt \"--END OF LINES\")\n" fo)
   ))
   (if solflg
    (progn
     (write-line "\n(prompt \"\\nSTART SOLID  LISTS--\")\n" fo)
     (prompt (strcat "\rWriting  SOLID DEFINITION POINTS to " nn "\n"))
     (setq
      llen (1- (length sollst))
      chklay (nth 0 sollst)  cnt 0  layflg T
     )
     (while (and layflg (< (setq cnt (1+ cnt)) llen))
      (setq layflg (= (nth cnt sollst) chklay))
     )
     (setq chkcol (nth 0 soclst)  cnt 0  colflg T)
     (while (and colflg (< (setq cnt (1+ cnt)) llen))
      (setq colflg (= (nth cnt soclst) chkcol))
     )
     (write-line "(mapcar" fo)
     (write-line 
      (strcat
       "'(lambda (" (if layflg "" "ly ") "p1 p2 p3 p4" (if colflg "" " cl")")"
      ) fo
     )
     (write-line "  (entmake" fo)
     (write-line
      (strcat
       "   (list\n   '(0 . \"SOLID\") "
       (if layflg (strcat "'(8 . " chklay ")") "(cons 8 ly)")
       "\n    (cons 10 p1) (cons 11 p2) (cons 12 p3) (cons 13 p4) "
       (if colflg (strcat "'(62 . " (itoa chkcol) ")") "(cons 62 cl)")
       "\n )))"
      )
      fo
     )
     (setq
      maplst (if layflg nil (list sollst))
      maplst
      (append maplst (list sp1lst)  (list sp2lst) (list sp3lst) (list sp4lst))
      maplst (if colflg maplst (append maplst (list soclst)))
     )
     (mapcar
     '(lambda (lst)
       (setq cnt -1  n 0  txlen 3)
       (princ "'(" fo)
       (while (setq itm (nth (setq cnt (1+ cnt)) lst))
        (setq fun (diddle fun))
        (if (and (/= cnt 0) (= n 0)) (princ " " fo))
        (setq n (1+ n))
        (cond
         ((= lst sollst)
          (setq txlen (+ txlen (strlen itm) 3))
          (if (< txlen 80)
           (princ (strcat (if (/= cnt 0) " " "") itm) fo)
           (progn
            (princ (strcat "\n  " itm) fo)
            (setq txlen (+ 2 (strlen itm)))
         )))
         (T
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (if (or (and (= n 16) (= lst soclst)) (and (= n 3) (/= lst soclst)))
           (progn
            (if (/= cnt llen) (write-line "" fo))
            (setq n 0)
       )))))
       (write-line (if (= n cnt) ")" "\n )") fo)
      )
      maplst
     )
     (write-line ")\n\n(prompt \"--END OF SOLIDS\")\n" fo)
   ))
   (if plflg
    (progn
     (write-line "\n(prompt \"\\nSTART PLINE  LISTS--\")\n" fo)
     (prompt (strcat "\rWriting  POLYLINE LISTS to " nn "\n"))
     (write-line "(foreach pollst" fo)
     (princ " (list" fo)
     (setq
      n -1
      n1 0
      llen (length plvlst)
      chklst '("t1chk" "t2chk" "blgchk" "cvchk" "flgchk") ;don't check points
      flglst '("t1flg" "t2flg" "blgflg" "cvflg" "flgflg")
     )
     (repeat 5          ; for each item on the check list
      (setq n1 (1+ n1))  ; increment counter
      (setq check        ; set check quantity for current vertex
       (set (read (nth (1- n1) chklst)) (nth 0 (nth (1+ n1) (nth 0 plvlst))))
      )
      (set (read (nth (1- n1) flglst)) T)  ; set flag true to show this property
      (setq lstcnt -1)                     ; is equal for all vertices
      (while             ; while current flag is true & there are still vertices
       (and
        (read (nth (1- n1) flglst))
        (< (setq lstcnt (1+ lstcnt)) (length plvlst))
       )
       (setq vtlst (nth lstcnt plvlst))  ; extract current vertex properties
       (setq n2 0)                       ; set counter for vertex properties
       (while                            ; while current flag is true and
        (and                             ; vertex property is not nil
         (read (nth (1- n1) flglst))     ; set flag nil if a vertex has unequal
         (setq tmp (nth (setq n2 (1+ n2)) (nth (1+ n1) vtlst))) ;of this property
        )
        (set (read (nth (1- n1) flglst)) (= tmp check))
      ))
     )
     (setq n -1)
     (repeat llen
      (princ "\n '(" fo)
      (setq cnt -1)
      (setq pvlst (nth (setq n (1+ n)) (reverse plvlst)))  ;extract each pline
      (foreach subpvl pvlst                                ;and vertex data
       (if
        (or
         (= (setq cnt (1+ cnt)) 1)
         (and (> cnt 1) (not (read (nth (- cnt 2) flglst))))
        )
        (princ "\n   " fo)
       )
       (cond
        ((= 0 cnt)                  ;PLINE
         (princ "(" fo)
         (repeat 4                  ;0, 8, 10, 40
          (princ (car subpvl) fo)
          (princ " " fo)
          (setq subpvl (cdr subpvl))
         )
         (princ "\n    " fo)
         (repeat (length subpvl)    ;41 - end
          (princ (car subpvl) fo)
          (setq subpvl (cdr subpvl))
         )
         (princ "\n   )" fo)
        )
        ((= 1 cnt)                  ;points of vertices
         (setq ct1 -1  ct2 0)
         (princ "(" fo)
         (while (setq itm (nth (setq ct1 (1+ ct1)) subpvl))
          (if (/= ct1 0) (princ " " fo))
          (princ itm fo)
          (setq ct2 (1+ ct2))
          (if (= ct2 3)
           (progn
            (princ "\n   " fo)
            (setq ct2 0)
         )))
         (if (= ct2 0)
          (princ ")" fo)
          (princ "\n   )" fo)
        ))
        (T                          ;other properties of vertices
         (if (not (eval (read (nth (- cnt 2) flglst))))
          (progn
           (setq ct1 -1  ct2 0)
           (princ "\n   (" fo)
           (while (setq itm (nth (setq ct1 (1+ ct1)) subpvl))
            (if (/= ct1 0) (princ " " fo))
            (princ itm fo)
            (setq ct2 (1+ ct2))
            (if (or (and (= ct2 8) (member cnt '( 2 3)))  (= ct2 16))
             (progn
              (princ "\n   " fo)
              (setq ct2 0)
           )))
           (princ ")" fo)
      )))))
      (princ "\n  )" fo)
     )
     (setq n nil)
     (write-line "\n )" fo)
     (write-line " (entmake (car pollst))" fo)
     (write-line " (mapcar" fo)
     (write-line
      (strcat    " '(lambda (sp" (if t1flg "" " t1") (if t2flg "" " t2")
       (if blgflg "" " blg") (if cvflg "" " cv") (if flgflg "" " fl") ")"
      ) fo
     )
     (write-line "   (entmake" fo)
     (write-line
      (strcat
       "    (list" 
       "\n    '(0 . \"VERTEX\") (cons 10 sp)"
       (if t1flg  (strcat " '(40 . " (rtos t1chk 2 3) ")")  " (cons 40 t1)")
       (if t2flg  (strcat " '(41 . " (rtos t2chk 2 3) ")")  " (cons 41 t2)")
       "\n     "
       (if blgflg (strcat " '(42 . " (rtos blgchk 2 3) ")") " (cons 42 blg)")
       (if cvflg  (strcat " '(50 . " (itoa cvchk) ")")  " (cons 50 cv)")
       (if flgflg (strcat " '(70 . " (itoa flgchk) ")") " (cons 70 fl)")
       "\n  )))"
      )
      fo
     )
     (write-line "  (car (setq pollst (cdr pollst)))   ;sp" fo)
     (if (not t1flg) (write-line "  (car (setq pollst (cdr pollst)))  ;t1" fo))
     (if (not t2flg) (write-line "  (car (setq pollst (cdr pollst)))  ;t2" fo))
     (if (not blgflg)(write-line "  (car (setq pollst (cdr pollst)))  ;rd" fo))
     (if (not cvflg) (write-line "  (car (setq pollst (cdr pollst)))  ;cf" fo))
     (if (not flgflg)(write-line "  (cadr pollst)   ;fl" fo))
     (write-line " )" fo)
     (write-line " (entmake '((0 . \"SEQEND\")))" fo)
     (write-line ")\n\n(prompt \"--END OF PLINES\")\n" fo)
   ))
   (if txtflg
    (progn
     (write-line "\n(prompt \"\\nSTART TEXT   LISTS--\")\n" fo)
     (prompt (strcat "\rWriting  TEXT LISTS to " nn "\n"))
     (setq
      llen (1- (length txlsty))
      chksty (nth 0 txlsty)  cnt 0  styflg T
     )
     (while (and styflg (< (setq cnt (1+ cnt)) llen))
      (setq styflg (= (nth cnt txlsty) chksty))
     )
     (setq
      chklay (nth 0 txllst)  cnt 0  layflg T)
     (while (and layflg (< (setq cnt (1+ cnt)) llen))
      (setq layflg (= (nth cnt txllst) chklay))
     )
     (setq chkjp (nth 0 txjlst)  cnt 0  jpflg T)
     (while (and jpflg (< (setq cnt (1+ cnt)) llen))
      (setq jpflg (= (nth cnt txjlst) chkjp))
     )
     (setq chkht (nth 0 txhlst)  cnt 0  htflg T)
     (while (and htflg (< (setq cnt (1+ cnt)) llen))
      (setq htflg (= (nth cnt txhlst) chkht))
     )
     (setq chkwd (nth 0 txwlst)  cnt 0  wdflg T)
     (while (and wdflg (< (setq cnt (1+ cnt)) llen))
      (setq wdflg (= (nth cnt txwlst) chkwd))
     )
     (setq chkang (nth 0 txalst)  cnt 0  angflg T)
     (while (and angflg (< (setq cnt (1+ cnt)) llen))
      (setq angflg (= (nth cnt txalst) chkang))
     )
     (setq chkjh (nth 0 tjhlst)  cnt 0  jhflg T)
     (while (and jhflg (< (setq cnt (1+ cnt)) llen))
      (setq jhflg (= (nth cnt tjhlst) chkjh))
     )
     (setq chkjv (nth 0 tjvlst)  cnt 0  jvflg T)
     (while (and jvflg (< (setq cnt (1+ cnt)) llen))
      (setq jvflg (= (nth cnt tjvlst) chkjv))
     )
     (write-line "(mapcar" fo)
     (write-line
      (strcat
       "'(lambda (txt " (if styflg "" "sty ") (if layflg "" "ly ")
       "sp" (if jpflg "" " jp") (if htflg "" " ht") (if wdflg "" " wd")
       (if angflg "" " ang") (if jhflg "" " jh") (if jvflg "" " jv")
       ")"
      ) fo
     )
     (write-line "  (entmake" fo)
     (write-line
      (strcat
       "   (list '(0 . \"TEXT\") (cons 1 txt) "
       (if styflg (strcat "'(7 . " chksty ") ")  "(cons 7 sty) ")
       (if layflg (strcat "'(8 . " chklay ")")  "(cons 8 ly)")
       "\n    (cons 10 sp) "
       (if  jpflg
        (strcat
         "'(11 " (rtos (nth 0 chkjp) 2 3) (rtos (nth 1 chkjp) 2 3)
         (rtos (nth 2 chkjp) 2 3) ") "
        )
        "(cons 11 jp) "
       )
       (if  htflg (strcat "'(40 . " (rtos chkht 2 3) ") ")  "(cons 40 ht) ")
       (if  wdflg (strcat "'(41 . " (rtos chkwd 2 3) ") ")  "(cons 41 wd) ")
       "\n   "
       (if angflg (strcat "'(50 . " (rtos chkang 2 3) ") ")" (cons 50 ang) ")
       (if  jhflg (strcat "'(72 . " (itoa chkjh)      ")")  "(cons 72 jh)")
       (if  jvflg (strcat "'(73 . " (itoa chkjv)      ")")  "(cons 73 jv)")
       "\n )))"
      )
      fo
     )
     (setq
      maplst (list txtlst)
      maplst (if styflg maplst (append maplst (list txlsty)))
      maplst (if layflg maplst (append maplst (list txllst)))
      maplst (append maplst (list txslst))
      maplst (if  jpflg maplst (append maplst (list txjlst)))
      maplst (if  htflg maplst (append maplst (list txhlst)))
      maplst (if  wdflg maplst (append maplst (list txwlst)))
      maplst (if angflg maplst (append maplst (list txalst)))
      maplst (if  jhflg maplst (append maplst (list tjhlst)))
      maplst (if  jvflg maplst (append maplst (list tjvlst)))
     )
     (mapcar
     '(lambda (lst)
       (setq cnt -1  n 0  txlen 2  endln nil)
       (princ "'(" fo)
       (while (setq itm (nth (setq cnt (1+ cnt)) lst))
        (setq fun (diddle fun))
        (if endln
         (progn
          (princ "\n " fo)
          (setq n 0)
        ))
        (setq n (1+ n))
        (cond
         ((or (= lst txtlst) (= lst txlsty) (= lst txllst))
          (setq 
           txlen (+ txlen (strlen itm) 3)
           endln nil
          )
          (if (< txlen 80)
           (princ (strcat (if (/= cnt 0) " " "") itm) fo)
           (progn
            (princ (strcat "\n  " itm) fo)
            (setq txlen (+ 2 (strlen itm)))
         )))
         ((or (= lst txslst) (= lst txjlst))
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq endln (and (/= cnt llen) (= n 3)))
         )
         ((or (= lst txhlst) (= lst txwlst) (= lst txalst))
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq endln (and (/= cnt llen) (= n 8)))
         )
         (T
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq endln (and (/= cnt llen) (= n 16)))
       )))
       (write-line (if (= n cnt) ")" "\n )") fo)
      )
      maplst
     )
     (write-line ")\n(prompt \"--END OF TEXT\")\n" fo)
   ))
   (if attflg
    (progn
     (write-line "\n(prompt \"\\nSTART ATTDEF LISTS--\")\n" fo)
     (prompt (strcat "\rWriting  ATTDEF LISTS to " nn "\n"))
     (setq
      llen (1- (length atdlst))
      chkdef (nth 0 atdlst)  cnt 0  defflg T
     )
     (while (and defflg (< (setq cnt (1+ cnt)) llen))
      (setq defflg (= (nth cnt atdlst) chkdef))
     )
     (setq chksty (nth 0 atlsty)  cnt 0  styflg T)
     (while (and styflg (< (setq cnt (1+ cnt)) llen))
      (setq styflg (= (nth cnt atlsty) chksty))
     )
     (setq chklay (nth 0 atllst)  cnt 0  layflg T)
     (while (and layflg (< (setq cnt (1+ cnt)) llen))
      (setq layflg (= (nth cnt atllst) chklay))
     )
     (setq chkjp (nth 0 atjlst)  cnt 0  jpflg T)
     (while (and jpflg (< (setq cnt (1+ cnt)) llen))
      (setq jpflg (= (nth cnt atjlst) chkjp))
     )
     (setq chkht (nth 0 athlst)  cnt 0  htflg T)
     (while (and htflg (< (setq cnt (1+ cnt)) llen))
      (setq htflg (= (nth cnt athlst) chkht))
     )
     (setq chkwd (nth 0 atwlst)  cnt 0  wdflg T)
     (while (and wdflg (< (setq cnt (1+ cnt)) llen))
      (setq wdflg (= (nth cnt atwlst) chkwd))
     )
     (setq chkang (nth 0 atalst)  cnt 0  angflg T)
     (while (and angflg (< (setq cnt (1+ cnt)) llen))
      (setq angflg (= (nth cnt atalst) chkang))
     )
     (setq chkvf (nth 0 atvlst)  cnt 0  vfflg T)
     (while (and vfflg (< (setq cnt (1+ cnt)) llen))
      (setq vfflg (= (nth cnt atvlst) chkvf))
     )
     (setq chkjh (nth 0 ajhlst)  cnt 0  jhflg T)
     (while (and jhflg (< (setq cnt (1+ cnt)) llen))
      (setq jhflg (= (nth cnt ajhlst) chkjh))
     )
     (setq chkjv (nth 0 ajvlst)  cnt 0  jvflg T)
     (while (and jvflg (< (setq cnt (1+ cnt)) llen))
      (setq jvflg (= (nth cnt ajvlst) chkjv))
     )
     (write-line "(mapcar" fo)
     (write-line
      (strcat
       "'(lambda ("(if defflg "" "def ") "tag prom " (if styflg "" "sty ")
       (if layflg "" "ly ") "sp" (if jpflg "" " jp") (if htflg "" " ht")
       (if wdflg "" " wd") (if angflg "" " ang") (if vfflg "" " vis")
       (if jhflg "" " jh") (if jvflg "" " jv") ")"
      ) fo
     )
     (write-line "  (entmake" fo)
     (write-line
      (strcat
       "   (list '(0 . \"ATTDEF\") "
       (if defflg (strcat "'(1 . " chkdef ") ")  "(cons 1 def) ")
       "(cons 2 tag) (cons 3 prom)\n    "
       (if styflg (strcat "'(7 . " chksty ")")  "(cons 7 sty)")
       (if layflg (strcat "'(8 . " chklay ")")  "(cons 8 ly)")
       " (cons 10 sp) "
       (if  jpflg
        (strcat
         "'(11 " (rtos (nth 0 chkjp) 2 3) (rtos (nth 1 chkjp) 2 3)
         (rtos (nth 2 chkjp) 2 3) ") "
        )
        "(cons 11 jp) "
       )
       (if  htflg (strcat "'(40 . " (rtos chkht 2 3) ") ")  "(cons 40 ht) ")
       (if  wdflg (strcat "'(41 . " (rtos chkwd 2 3) ") ")  "(cons 41 wd) ")
       "\n    "
       (if angflg (strcat "'(50 . " (rtos chkang 2 3) ") ")" (cons 50 ang) ")
       (if  vfflg (strcat "'(70 . " (itoa chkvf)      ")")  "(cons 70 vis) ")
       (if  jhflg (strcat "'(72 . " (itoa chkjh)      ")")  "(cons 72 jf)")
       (if  jvflg (strcat "'(74 . " (itoa chkjv)      ")")  "(cons 74 jf)")
       "\n )))"
      )
      fo
     )
     (setq
      maplst (if defflg nil (list atdlst))
      maplst (append maplst (list atglst) (list atplst))
      maplst (if styflg maplst (append maplst (list atlsty)))
      maplst (if layflg maplst (append maplst (list atllst)))
      maplst (append maplst (list atslst))
      maplst (if  jpflg maplst (append maplst (list atjlst)))
      maplst (if  htflg maplst (append maplst (list athlst)))
      maplst (if  wdflg maplst (append maplst (list atwlst)))
      maplst (if angflg maplst (append maplst (list atalst)))
      maplst (if  vfflg maplst (append maplst (list atvlst)))
      maplst (if  jhflg maplst (append maplst (list ajhlst)))
      maplst (if  jvflg maplst (append maplst (list ajvlst)))
     )
     (mapcar
     '(lambda (lst)
       (setq cnt -1  n 0 txlen 2  endln nil)
       (princ "'(" fo)
       (while (setq itm (nth (setq cnt (1+ cnt)) lst))
        (setq fun (diddle fun))
        (if endln
         (progn
          (princ "\n " fo)
          (setq n 0)
        ))
        (setq n (1+ n))
        (cond
         ((or
           (= lst atdlst) (= lst atglst) (= lst atplst)
           (= lst atlsty) (= lst atllst)
          )
          (setq 
           txlen (+ txlen (strlen itm) 3)
           endln nil
          )
          (if (< txlen 80)
           (princ (strcat (if (/= cnt 0) " " "") itm) fo)
           (progn
            (princ (strcat "\n  " itm) fo)
            (setq txlen (+ 2 (strlen itm)))
         )))
         ((or (= lst atslst) (= lst atjlst))
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq endln (and (/= cnt llen) (= n 3)))
         )
         ((or (= lst athlst) (= lst atwlst) (= lst atalst))
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq endln (and (/= cnt llen) (= n 8)))
         )
         (T
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (setq endln (and (/= cnt llen) (= n 16)))
       )))
       (write-line (if (= n cnt) ")" "\n )") fo)
      )
      maplst
     )
     (write-line ")\n(prompt \"--END OF ATTDEF\")" fo)
   ))
   (if ptflg
    (progn
     (write-line "\n(prompt \"\\nSTART POINT  LISTS--\")\n" fo)
     (prompt (strcat "\rWriting  POINT LISTS to " nn "\n"))
     (setq
      llen (1- (length ptllst))
      chklay (nth 0 ptllst)  cnt 0  layflg T
     )
     (while (and layflg (< (setq cnt (1+ cnt)) llen))
      (setq layflg (= (nth cnt ptllst) chklay))
     )
     (setq chkcol (nth 0 ptclst)  cnt 0  colflg T)
     (while (and colflg (< (setq cnt (1+ cnt)) llen))
      (setq colflg (= (nth cnt ptclst) chkcol))
     )
     (write-line "(mapcar" fo)
     (write-line
      (strcat
       "'(lambda (" (if layflg "" "ly ") "sp" (if colflg "" " cl") ")"
      ) fo
     )
     (write-line "  (entmake" fo)
     (write-line
      (strcat
       "   (list '(0 . \"POINT\") "
       (if layflg (strcat "'(8 . " chklay ")") "(cons 8 ly)")
       " (cons 10 sp) "
       (if colflg (strcat "'(62 . " (itoa chkcol) ")") "(cons 62 cl)")
       ")\n ))"
      )
      fo
     )
     (setq
      maplst (if layflg nil (list ptllst))
      maplst (append maplst (list ptslst))
      maplst (if colflg maplst (append maplst (list ptclst)))
     )
     (mapcar
     '(lambda (lst)
       (setq cnt -1  n 0  txlen 2)
       (princ "'(" fo)
       (while (setq itm (nth (setq cnt (1+ cnt)) lst))
        (setq fun (diddle fun))
        (if (and (/= cnt 0) (= n 0)) (princ " " fo))
        (setq n (1+ n))
        (cond
         ((= lst ptllst)
          (setq txlen (+ txlen (strlen itm) 3))
          (if (< txlen 80)
           (princ (strcat (if (/= cnt 0) " " "") itm) fo)
           (progn
            (princ (strcat "\n  " itm) fo)
            (setq txlen (+ 2 (strlen itm)))
         )))
         (T
          (if (/= cnt 0) (princ " " fo))
          (princ itm fo)
          (if (or (and (= n 16) (= lst ptclst)) (and (= n 3) (/= lst ptclst)))
           (progn
            (if (/= cnt llen) (write-line "" fo))
            (setq n 0)
       )))))
       (write-line (if (= n cnt) ")" "\n )") fo)
      )
      maplst
     )
     (write-line ")\n\n(prompt \"--END OF POINTS\")\n" fo)
   ))
   (write-line "(prompt \"\\n--END OF FILE--\")" fo)
   (write-line "(princ)" fo) ;add (princ) for clean end
   (setq fo (close fo))
   (prompt (strcat "\rFile " nn " complete!"))
 ))
 (princ)
)


; XTRDXF - extract association from string list

(defun xtrdxf (strg dxfno)
 (cdr
  (read
   (strcat
    (stres2
     (substr strg (instr strg (strcat "(" (itoa dxfno) " ")))
     ")"
    )
    ")"
 )))
)


; List of Called Routines
; ASSOC1-SORT  DIDDLE  DXF  INSTR  PRETTY  STRES2  UINT


; INSTR - find the location of a string (str2) in another string(str1)

(defun instr (str1 str2 / no lstr1 lstr2 currnt)
 (setq currnt 0 no nil)
 (if (= str1 str2)
  (setq no 1)
  (if (and str1 str2)
   (if (>= (setq lstr1 (strlen str1))(setq lstr2 (strlen str2)))
    (while (<= (setq currnt (1+ currnt)) (1+ (- lstr1 lstr2)))
     (if (= str2 (substr str1 currnt lstr2)) (setq no currnt currnt lstr1))
 ))))
 no
)



; STRES2 - returns portion of S1 ending before S2 or nil

(defun stres2 (s1 s2 / l1 l2 pos)
 (setq
  l1  (strlen s1)
  l2  (strlen s2)
  pos (instr s1 s2)
 )
 (cond
  ((not pos) nil)
  ((<= l1 l2) nil)
  ((= (substr s1 pos l2) s2) (substr s1 1 (1- pos)))
 )
)


;PRETTY - pad strings to desired length with character of choice
;       - s : string to pad  len : length required  pad : padding char.
;       - flag : T for after, nil for in front

(defun pretty (s len pad flag)
 (while (< (strlen s) len) (setq s (if flag (strcat s pad) (strcat pad s))))
 s
)


; DXF - extract DXF code from list

(defun dxf (code elist) (cdr (assoc code elist)))


; DIDDLE - show progress

(defun diddle (fun)
 (if (not fun)(setq fun 0))
 (cond
  ((= fun 0) (prompt "\r/    /"))
  ((= fun 1) (prompt "\r-    -"))
  ((= fun 2) (prompt "\r\\    \\"))
  ((= fun 3) (prompt "\r|    |") (setq fun -1))
 )
 (setq fun (1+ fun))
)

; UANGLE, UDIST, UENTSL, UINT, UKWORD, UPOINT, UREAL, USTR 
; - provide formatting and default values for data entry
; These functions are modified from a suite written by
; Rustin Gesner & Joseph Smith and included in their book
; "Maximizing AutoLISP" published by New Riders Publishing.

(defun uint (bit kwd msg def / inp)
 (setq
  msg (strcat "\n" msg (if def (strcat " <" (itoa def) ">") "") ": ")
  bit (if def (- bit (logand bit 1)) bit)
 )
 (if kwd (initget bit kwd) (initget bit))
 (if (setq inp (getint msg)) inp def)
)


;ref - CADENCE May 1993 - Leif Eriksen

; ASSOC1-SORT - association list sort on 1st element

(defun assoc1-sort (x)
 (setq cmp assoc1-cmp)
 (sort x T)
)


; SORT - main sort routine

(defun sort (x dir)
 (cond
  ((null (cdr x)) x)
  (T (merge (sort (first-half x) dir) (sort (last-half x) dir) dir))
 )
)


; MERGE - main routine to rejoin sorted parts of lists

(defun merge (a b dir)
 (cond
  ((null a) b)
  ((null b) a)
  ((< (cmp (car a) (car b) dir) 0)  (cons (car a) (merge (cdr a) b dir)))
  (T (cons (car b) (merge a (cdr b) dir)))
 )
)


; FIRST-HALF - construct 1st half of list

(defun first-half (l)
 (head l (1- (length l)))
)


;LAST-HALF - construct 2nd half of list

(defun last-half (l)
 (tail l (1- (length l)))
)


; HEAD - reduce list to front

(defun head (l n)
 (cond
  ((minusp n) nil)
  (T (cons (car l) (head (cdr l) (- n 2))))
 )
)


; TAIL - reduce list to end

(defun tail (l n)
 (cond
  ((minusp n) l)
  (T (tail (cdr l) (- n 2)))
 )
)




; ASSOC1-CMP - compare 1st no. (X) in sublist of assoc. list

(defun assoc1-cmp (a b dir)
 (cond
  ((< (car a) (car b)) (if dir -1  1))
  ((> (car a) (car b)) (if dir  1 -1))
  ((= (car a) (car b))  0)
 )
)




(princ)
